--[[

--            MANY MINOR TWEAKS v6            --
    This is yet again another big revision for
    Many Minor Tweaks. This one aims to bring
    bring another new menu system to hopefully
     address any remaining issues related to
     netplay, along with just being a better
                 interface to use.
    There is also intensive behind-the-scenes
     updates to this, to bring my own code up
          to better standards, I hope!

]]

--[[
         --] addTweak Legend [---
    * name: Name used for menus.
    * intname: Internal name, used for saving and what variables to check.
    * menu: What menu should this will be added to.
    * options: "boolean" for boolean (will be literal true or false), "number" for multiple-choice numbers (0 should be assumed to be "off") that are between -9 and 61. "bignumber" will be for numbers bigger than 61.
        * range: Range of numbers that will be used (allowed values are from -9 to 61)
        * names: List of table for the names associated with each number, used for the menu.
    * default: What the default option is, duh.
    * descriptiont: Description table, table of strings to use for a description.
    * func: Functions to be ran inside a playerthink for any given player that has this option enabled.
        * prethinkfunc: Prethink version of this.
        * postthinkfunc: Ditto, but for postthink.
    * dontresetme: Do not reset all the internal variables upon level reset.
    * global: Global variable, used by tweaks that aren't player-specific. This will be synced using NetVars. This also will make functions run under "ThinkFrame" instead of per-player.

    Plans:
    * import: File to check to import settings from the previous version of MMT.
    * subsetting: This will set itself underneath a specific setting in the menu, making sure it doesn't get it's own unique name.

]]

local mmtKeys = {"off", "custom1", "custom2", "custom3", "spin", "jump", "tossflag", "attack", "attacknormal", "nextwep", "prevwep"}
local mmtKeysRange = {0, 10}

if SUBVERSION >= 14 or VERSION >= 203 then
    table.insert(mmtKeys, "shield")
    mmtKeysRange = {0, 11}
end

MMT.mmtKeys = mmtKeys

MMT.addTweak{name = "Special Stage Ring Animation", intname = "ssra", menu = "enhance", options = "boolean", default = true,
descriptiont = {"When exiting a stage with an emerald token, you get an ", "animation for warping through a special stage!"},
func = function(key, player, selft)
    if not selft.SSE and player.exiting > 0 and ((token > 0  and (player.pflags & PF_FINISHED)) or (G_IsSpecialStage(gamemap) and player.exiting < 43)) then
        selft.SSE = true
        P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_MMTSPECIALSTAGEENT)
        S_StartSound(player.mo, sfx_s3kb3)
    end

    if selft.SSE and not (mapheaderinfo[gamemap].typeoflevel & TOL_NIGHTS) then
        player.mo.momx = 0
        player.mo.momy = 0
        player.mo.momz = 0
        if not player.mmtSettings.pshield or (player.mmtStorage["pshield"] and player.mmtStorage["pshield"].ShieldAfterLevel) then
            player.powers[pw_shield] = 0
        end
        player.mo.state = S_PLAY_STND
        player.mo.flags2 = $ | MF2_DONTDRAW
        S_StopSoundByID(player.mo, sfx_token)
    end
end
}

MMT.addTweak{name = "Classic Shields", intname = "classicshield", menu = "misctweaks", options = "boolean", default = false, global = true,
descriptiont = {"Replaces all shield monitors in levels with similar shields from Sonic 3!"}
}

MMT.addTweak{name = "Spin Steering", intname = "spinsteer", menu = "gametweaks", options = "number", default = 0, range = {0, 20},
descriptiont = {"Adjusts your ability to steer while spinning on the ground."},
func = function(key, player, selft)
    if (player.pflags & PF_SPINNING) and not (player.pflags & PF_STARTDASH) and not (player.pflags & PF_JUMPED) and (player.panim == PA_ROLL)
    and (P_IsObjectOnGround(player.mo) or player.mmtSettings.spstmidair > 0) then
        if not selft.StoredThing then
            selft.StoredThing = player.thrustfactor
        end
        local Value = player.mmtSettings.spinsteer
        if not P_IsObjectOnGround(player.mo) and player.mmtSettings.spstmidair != 3 then
            if player.mmtSettings.spstmidair == 1 then
                Value = $/2
            else
                Value = $/2 + Value/3
            end
        end
        player.thrustfactor = (skins[player.mo.skin].thrustfactor)*Value
    elseif selft.StoredThing then
        player.thrustfactor = selft.StoredThing
        selft.StoredThing = nil
    end
end
}

MMT.addTweak{name = "Spin Steer Air Control", intname = "spstmidair", menu = "gametweaks", options = "number", default = 0, range = {0, 3}, names = {"off", "lightest", "lighter", "normal"}, subsetting = "spinsteer",
descriptiont = {"How much should you be able to control your spinning in mid-air?", "(this will be affected by the main Spin Steering option)"}
}

MMT.addTweak{name = "Preserve Shield", intname = "pshield", menu = "gametweaks", options = "number", default = 0, range = {0, 2}, names = {"off", "Between acts", "Between acts & zones"},
dontresetme = true,
descriptiont = {"Preserve your shield to the next level, or zone!"},
func = function(key, player, selft)
    if G_EnoughPlayersFinished() then
        if not selft.ShieldAfterLevel and not G_IsSpecialStage(gamemap) then
            selft.ShieldAfterLevel = player.powers[pw_shield]
            selft.ShieldLevelName = mapheaderinfo[gamemap].lvlttl
        end
    elseif selft.ShieldAfterLevel and not G_IsSpecialStage(gamemap) then
        if (selft.ShieldLevelName == mapheaderinfo[gamemap].lvlttl) or player.mmtSettings["pshield"] == 2 then
            player.powers[pw_shield] = selft.ShieldAfterLevel
            P_SpawnShieldOrb(player)
        end
        selft.ShieldAfterLevel = nil
    end
end
}

MMT.addTweak{name = "Remove Roll Lock", intname = "nrl", menu = "gametweaks", options = "number", default = 0, range = {0, 2}, names = {"off", "Only after jumping", "At all times"},
descriptiont = {"Allows you to have greater control when you're in 'rolling' mode, ", "and you leave the ground."},
func = function(key, player, selft)
    if not P_IsObjectOnGround(player.mo) and (player.pflags & PF_SPINNING)
    and (((player.pflags & PF_JUMPED) and player.mmtSettings.nrl == 1) or player.mmtSettings.nrl == 2)
    then
        player.pflags = $ & ~PF_SPINNING
        player.mo.state = S_PLAY_JUMP
    end
end
}

MMT.addTweak{name = "Activate Menu w/ Button", intname = "amwb", menu = "misctweaks", options = "number", default = 3, range = mmtKeysRange, names = mmtKeys,
descriptiont = {"Hold down this button for 3 seconds to bring up the MMT Menu!"},
func = function(key, player, selft)
    if player.mmtMenu.menu then return end
    if player.mmtbt[mmtKeys[player.mmtSettings.amwb + 1]] >= TICRATE and player.mmtbt[mmtKeys[player.mmtSettings.amwb + 1]] < 4*TICRATE then
        if (player.mmtbt[mmtKeys[player.mmtSettings.amwb + 1]] % TICRATE) == 0 then
            S_StartSound(nil, sfx_menu1, player)
        end
    end

    if player.mmtbt[mmtKeys[player.mmtSettings.amwb + 1]] == 4*TICRATE then
        MMT.SetPlayerMenu(player, "mainmenu", "mainmenu")

        local musictrack = "MMTM"
        if player.mmtSettings.menum == 0 then
            return
        elseif player.mmtSettings.menum == 1 then
            musictrack = MMT.MusicList[P_RandomRange(1, #MMT.MusicList)]
        else
            musictrack = MMT.MusicList[player.mmtSettings.menum - 1]
        end
        S_ChangeMusic(musictrack, nil, player)
    end
end
}

MMT.addTweak{name = "Falling Animation", intname = "saw", menu = "enhance", options = "boolean", default = true,
descriptiont = {"Stops your character from walking when falling in the air!"},
func = function(key, player, selft)
    if ((player.mmtSettings.coyote and player.mo.coyoteTime == 0) or (not player.mmtSettings.coyote)) -- just something to make things seem smoother
    then
        if not(P_IsObjectOnGround(player.mo))
        and (player.mo.state == S_PLAY_WALK or player.mo.state == S_PLAY_RUN)
        and not player.powers[pw_carry] then
            selft.FallingAnimation = true
            player.mo.state = S_PLAY_FALL
        end
    else
        selft.FallingAnimation = nil
    end

    if P_IsObjectOnGround(player.mo) then
        selft.FallingAnimation = nil
    end
end
}

MMT.addTweak{name = "Deaths Counter", intname = "deathscounter", menu = "gametweaks", options = "number", default = 0, range = {0, 2}, names = {"off", "on", "Reset after level ends"},
descriptiont = {"Replaces your lives counter with a deaths counter. ", "This effectively gives you infinite lives."}
}

MMT.addTweak{name = "Coyote Time", intname = "coyote", menu = "gametweaks", options = "boolean", default = false,
descriptiont = {"Gives you a brief period of time after falling off a ledge to allow for a jump."}
}

MMT.addTweak{name = "Recovery Jump", intname = "recovery", menu = "gametweaks", options = "boolean", default = false,
descriptiont = {"Allows you to jump shortly after you take damage."}
}

MMT.addTweak{name = "Spring Twirl", intname = "springtwirl", menu = "enhance", options = "boolean", default = false,
descriptiont = {"Makes your character twirl in the air when they touch a spring, ", "just like in Sonic CD!"}
}

MMT.addTweak{name = "Decrease Ring Scattering", intname = "ringscatter", menu = "gametweaks", options = "boolean", default = false,
descriptiont = {"Rings scatter about the same amount regardless of how many times you're hit."},
func = function(key, player, selft)
    player.losstime = 0
end}

local function EarliestEmerald()
    if not (emeralds & EMERALD1) then
        return 0
    elseif not (emeralds & EMERALD2) then
        return 1
    elseif not (emeralds & EMERALD3) then
        return 2
    elseif not (emeralds & EMERALD4) then
        return 3
    elseif not (emeralds & EMERALD5) then
        return 4
    elseif not (emeralds & EMERALD6) then
        return 5
    elseif not (emeralds & EMERALD7) then
        return 6
    end
end

MMT.addTweak{name = "Tails Hold 2 Fly", intname = "th2f", menu = "gametweaks", options = "boolean", default = true,
descriptiont = {"Instead of having to mash Jump, you can just hold it to fly as Tails."},
func = function(key, player, selft)
    if (player.powers[pw_tailsfly] > 0) and player.mmtbt.jump > 0 then
        player.fly1 = 1
    end
end
}

MMT.addTweak{name = "Super 4 All", intname = "s4a", menu = "gametweaks", options = "boolean", default = true,
descriptiont = {"Allows all characters to have super forms."},
func = function(key, player, selft)
    player.charflags = $1 | SF_SUPER
end
}

MMT.addTweak{name = "Cancel Super", intname = "cancelsoup", menu = "gametweaks", options = "number", default = 0, range = mmtKeysRange, names = mmtKeys,
descriptiont = {"Let's you cancel your character's super form with this button."},
func = function(key, player, selft)
    if player.powers[pw_super] and (player.mmtbt[mmtKeys[player.mmtSettings.cancelsoup + 1]] == 1) and not (player.mo.state >= S_PLAY_SUPER_TRANS1 and player.mo.state <= S_PLAY_SUPER_TRANS6) then
        player.pflags = $1 | PF_THOKKED
        player.powers[pw_flashing] = TICRATE
        player.powers[pw_super] = 0
        player.mo.color = player.skincolor
        P_RestoreMusic(player)
        P_SpawnShieldOrb(player)
    end
end}

MMT.addTweak{name = "Total Rings", intname = "ringcount", menu = "enhance", options = "boolean", default = true,
descriptiont = {"Open the scoreboard to see how many rings in the level you've collected, ", "compared to the total amount!"},
}

MMT.addTweak{name = "Super Sparkles", intname = "sspark", menu = "enhance", options = "boolean", default = true,
descriptiont = {"Gives invincibility sparkles to your character when super."},
func = function(key, player, selft)
    if player.powers[pw_super] and leveltime % (TICRATE/7) == 0
    and not (player.mo.state >= S_PLAY_SUPER_TRANS1 and player.mo.state <= S_PLAY_SUPER_TRANS6)
    then
        local sparks = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_IVSP)
        sparks.scale = player.mo.scale
    end
end
}

MMT.addTweak{name = "Super Trail", intname = "strail", menu = "enhance", options = "boolean", default = true,
descriptiont = {"Adds a ghost trail behind you when you're super."},
func = function(key, player, selft)
    if player.powers[pw_super] and not player.mo.hyperflashcolor -- prevents conflict with hyper abilities
    and not (player.speed < 5*FRACUNIT and player.mo.momz == 0) then
        local traily = P_SpawnGhostMobj(player.mo)
        traily.fuse = 4
        traily.rollangle = player.mo.rollangle
        if (traily.tracer) then
            traily.tracer.fuse = 4
        end
    end
end
}

MMT.addTweak{name = "Always Render Shields", intname = "alwaysshield", menu = "enhance", options = "boolean", default = false,
descriptiont = {"Always show the player's shield in conditions where it normally wouldn't show up."},
func = function(key, player, selft) -- it seems there's very very mild hitching using this method, and using pre or post think doesn't fix that. oh well.
    if not selft.FakeShield and player.playerstate == PST_LIVE then
        selft.FakeShield = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_FAKE_SHIELD)
        selft.FakeShield.target = player.mo
    end
    if selft.FakeShield and selft.FakeShield.valid then
        local shield = selft.FakeShield
        if (player.powers[pw_super] or player.powers[pw_invulnerability]) and shield.DontRender then
            shield.DontRender = false
            shield.SetState = false
        end
    end

    if player.playerstate == PST_DEAD and selft.FakeShield and selft.FakeShield.valid then
        selft.FakeShield.TerminateSelf = true
        selft.FakeShield = nil
    end
end
}



MMT.addTweak{name = "No Super Music", intname = "nosoup", menu = "enhance", options = "boolean", default = false,
descriptiont = {"Disables the music when you go super from playing."},
func = function(key, player, selft) -- it seems there's very very mild hitching using this method, and using pre or post think doesn't fix that. oh well.
    if S_MusicName(player) == "_super" or (player.mo.skin == "xsonic" and S_MusicName(player) == "XSSS") then
        S_ChangeMusic(selft.musicnow, nil, player, nil, selft.musicpos)
    end

    --print(S_MusicName(player))

    if S_MusicName(player) != "_inv" and S_MusicName(player) != "_shoes" and S_MusicName(player) != "_1up" then
        selft.musicnow = S_MusicName(player)
        selft.musicpos = S_GetMusicPosition()
    end
end
}

MMT.addTweak{name = "No Super Hover", intname = "nosuperhover", menu = "gametweaks", options = "boolean", default = false,
descriptiont = {"Disables Super Sonic's ability to hover when holding Spin in mid-air."},
}

MMT.addTweak{name = "Angle Stand", intname = "AngleStand", menu = "enhance", options = "boolean", default = false,
descriptiont = {"Stops character from automatically turning towards the current camera angle."},
}

MMT.addTweak{name = "AS Strafe Mode", intname = "strafeAnglestand", menu = "enhance", options = "boolean", default = true, subsetting = "AngleStand",
descriptiont = {"Autoturn cancelling applies to strafe mode."},
}

addHook("JumpSpinSpecial", function(player)
    if player.powers[pw_super] and player.charability == CA_THOK and player.mmtSettings.nosuperhover then
        return true
    end
end)

MMT.addTweak{name = "Speedy Particles", intname = "sp", menu = "enhance", options = "boolean", default = false,
descriptiont = {"Gives you particles when your character is up to speed."},
func = function(key, player, selft)
    if P_IsObjectOnGround(player.mo) and (player.speed > player.runspeed) then
        local part = MMT.P_SpawnCoolSkidDust(player, player.mo.radius/FRACUNIT, false)
        part.destscale = player.mo.scale/6+player.mo.scale/5
        part.scalespeed = FRACUNIT/19
    end
    --FixedMul(player.runspeed, 2*FRACUNIT+FixedDiv(3*FRACUNIT, 4*FRACUNIT))
    if player.mmtSettings.spf then
        if P_IsObjectOnGround(player.mo) and not (player.mo.eflags & (MFE_TOUCHWATER|MFE_UNDERWATER)) and
        (player.speed > 55*FRACUNIT or (player.speed > 40*FRACUNIT and (player.powers[pw_sneakers] or player.dashmode >= 3*TICRATE))) then
            local part2 = MMT.P_SpawnCoolSkidDust(player, player.mo.radius/FRACUNIT - ((player.mo.radius/FRACUNIT) / 6), false)
            part2.state = S_SPINDUST_FIRE1
            part2.destscale = player.mo.scale/2+player.mo.scale/11
            part2.scalespeed = FRACUNIT/30
            part2.tics = TICRATE
        end
    end
end
}

MMT.addTweak{name = "Flame Particles", intname = "spf", menu = "enhance", options = "boolean", default = false, subsetting = "sp",
descriptiont = {"Turns your particles into flames when you're going REALLY fast!"},
}

MMT.addTweak{name = "Fade Music", intname = "fademusic", menu = "enhance", options = "boolean", default = true,
descriptiont = {"Fades the music out when you finish a level."},
func = function(key, player, selft)
    if not selft.MusicVolume then
        selft.MusicVolume = 100
    end

    if G_EnoughPlayersFinished() then
        if selft.MusicVolume > 0 then
            selft.MusicVolume = $ - 1
        end
        S_SetInternalMusicVolume(selft.MusicVolume, player)
    end
end
}

MMT.addTweak{name = "Run on Water", intname = "waterrunner", menu = "gametweaks", options = "boolean", default = false,
descriptiont = {"Allows your character to run on water!"},
func = function(key, player, selft)
    if abs(player.mo.momz) < FRACUNIT and (player.speed >= player.runspeed + (player.mmtSettings.rowreq*FRACUNIT)) then
        player.charflags = $1|SF_RUNONWATER
    else
        player.charflags = $1 & ~SF_RUNONWATER
    end
end
}

MMT.addTweak{name = "RoW Speed", intname = "rowreq", menu = "gametweaks", options = "number", default = 0, range = {0, 30}, subsetting = "waterrunner",
descriptiont = {"Additional speed needed to run on water"},
}

MMT.addTweak{name = "Token to Rings", intname = "tokenring", menu = "gametweaks", options = "boolean", default = true,
descriptiont = {"When you have all 7 emeralds, Emerald Tokens give you ", "50 rings instead of a continue."},
}

addHook("TouchSpecial", function(emeraldtoken, toucher)
    if toucher.type == MT_PLAYER then
        local player = toucher.player
        if player.mmtSettings.tokenring and All7Emeralds(emeralds) then
            P_GivePlayerRings(player, 50)
            P_AddPlayerScore(player, 1000)
            S_StartSound(player.mo, sfx_cdfm66)
            emeraldtoken.flags = $ & ~MF_SPECIAL
            P_KillMobj(emeraldtoken)
            return true
        else
            return false
        end
    end
end, MT_TOKEN)

MMT.addTweak{name = "Tails Drop Ball", intname = "tdb", menu = "gametweaks", options = "number", default = 0, range = mmtKeysRange, names = mmtKeys,
descriptiont = {"Allow Tails to curl up into a ball mid-flight using this key."},
func = function(key, player, selft)
    if (((player.charability == CA_FLY) and (player.charability2 == CA2_SPINDASH) and (skins[player.mo.skin].name != "eggman")) or player.mmtSettings.forcetb)
    and player.panim == PA_ABILITY and (player.mmtbt[mmtKeys[player.mmtSettings.tdb + 1]] == 1) then
        S_StartSound(player.mo, sfx_3db16)
        player.mo.state = S_PLAY_JUMP
        player.pflags = $1 | PF_JUMPED
        player.pflags = $1 | PF_THOKKED
        player.pflags = $ & ~PF_CANCARRY
        selft.tailsball = 1
        selft.tailsfly = player.powers[pw_tailsfly]
        selft.bop = false
    end

    if selft.tailsball then
        selft.tailsball = $ + 1
    end

    if P_IsObjectOnGround(player.mo) then
        selft.tailsball = 0
        selft.tailsfly = 0
        selft.bop = false
    end

    selft.tailsball = $ or 0

    if selft.bop then -- cap tails' flight power; i'd ideally maybe award a little bit of time for successful bops, but it'd be hard to figure that out.
        player.powers[pw_tailsfly] = min($, selft.tailsfly)
    end

    if player.mmtSettings.tdbp and selft.tailsball > TICRATE+1 and not selft.bop then
        player.pflags = $ & ~PF_THOKKED
        selft.bop = true
    end
end}

MMT.addTweak{name = "TDB Plus", intname = "tdbp", menu = "gametweaks", options = "boolean", default = false, subsetting = "tdb",
descriptiont = {"Allow actions again after a second has passed ", "from using \"Tails Drop Ball\"."}
}

MMT.addTweak{name = "Force Tails Drop Ball", intname = "forcetb", menu = "compattweaks", options = "boolean", default = false, subsetting = "tdb",
descriptiont = {"If your character has the ability to fly, ", "but Tails Drop Ball isn't working, you can force it to work."}
}

MMT.addTweak{name = "Instant Respawn", intname = "instantrespawn", menu = "gametweaks", options = "boolean", default = false,
descriptiont = {"When you die, you instantly respawn."},
func = function(key, player, selft)
    if (gametype == GT_COOP) and player.playerstate == PST_DEAD then
        player.playerstate = PST_REBORN
    end
end
}

MMT.addTweak{name = "Uncurl", intname = "uncurl", menu = "gametweaks", options = "boolean", default = false,
descriptiont = {"Lets you uncurl when you're going fast enough on the ground."},
func = function(key, player, selft)
    if selft.denyuncurl and player.mmtbt.spin == 0 then
        selft.denyuncurl = false
    end

    if P_IsObjectOnGround(player.mo)
    and (player.pflags & PF_SPINNING)
    and (player.speed > player.runspeed - player.runspeed/3)
    and not selft.denyuncurl
    and player.mmtbt.spin == 1
    then
        player.pflags = $ & ~PF_SPINNING
        player.mo.state = S_PLAY_WALK
        S_StartSound(player.mo,sfx_3db16)
    end
end,
prethinkfunc = function(key, player, selft)
    if player.mmtbt.spin > 0 and (player.pflags & PF_SPINNING) == 0 then
        selft.denyuncurl = true
    end
end
}

MMT.addTweak{name = "Recurl", intname = "recurl", menu = "gametweaks", options = "number", default = 0, range = mmtKeysRange, names = mmtKeys,
descriptiont = {"Allows you to re-curl up into a ball after using a spring, using this button! ", "Set the plus options to allow mid-air actions to be performed too."},
func = function(key, player, selft)
    if player.panim == PA_SPRING then --player.mo.sprite2 == SPR2_SPNG then
        selft.UsedSpring = true
        selft.GrantedReability = false
    elseif (player.panim == PA_PAIN and not P_PlayerInPain(player)) then
        selft.UsedTornado = true
        selft.GrantedReability = false
    elseif not selft.UsedSpring and not selft.UsedTornado and player.panim == PA_FALL and not (player.pflags & PF_SHIELDABILITY) and not (player.charability == CA_FLOAT and (player.pflags & PF_THOKKED)) then
        selft.UsedMisc = true
        selft.GrantedReability = false
    end
    if not P_IsObjectOnGround(player.mo)
    and ((selft.UsedSpring or selft.UsedMisc) or (selft.UsedTornado and (player.panim == PA_FALL)))
    and (player.mmtbt[mmtKeys[player.mmtSettings.recurl + 1]])
    and ((player.charability2 == CA2_SPINDASH) or player.mmtSettings.forcerc)
    and player.secondjump == 0 -- prevents weirdness with super sonic
    and not selft.GrantedReability
    --and (not player.recurlwait or (selft.UsedSpring and selft.UsedTornado and player.panim == PA_FALL))
    and player.mo.state != S_PLAY_JUMP then
        player.mo.state = S_PLAY_JUMP
        player.pflags = $1 | PF_JUMPED
        if (not player.mmtSettings.rcp) or (player.mmtSettings.saw and player.mmtStorage.saw.FallingAnimation)  then
            player.pflags = $1 | PF_THOKKED
        elseif not selft.GrantedReability then
            selft.GrantedReability = true
            player.pflags = $ & ~PF_SHIELDABILITY
            player.pflags = $ & ~PF_THOKKED
        end
        S_StartSound(player.mo, sfx_spin, player)
        --if player.MMTrecurlmulti != "on" then player.recurlwait = true end
    end

    if P_IsObjectOnGround(player.mo) then
        selft.UsedSpring = false
        selft.UsedTornado = false
        selft.UsedMisc = false
        selft.GrantedReability = false
    end
end
}

MMT.addTweak{name = "Recurl Plus", intname = "rcp", menu = "gametweaks", options = "boolean", default = false, subsetting = "recurl",
descriptiont = {"Allows you to use actions (thokking, flying, etc.) again after utilizing Recurl."}
}

MMT.addTweak{name = "Force Recurl", intname = "forcerc", menu = "compattweaks", options = "boolean", default = false, subsetting = "recurl",
descriptiont = {"If your character doesn't have the proper character flags, ", "you can force Recurl to work for them anyways."}
}

MMT.addTweak{name = "Menu Music", intname = "menum", menu = "misctweaks", options = "number", default = 1, range = {0, 6}, names = {"off", "Random", "Pro Baseball News", "Load", "For Your Convienence", "Lobby Line", "File Select"},
descriptiont = {"Change the music played when you're in the menus!"},
}

MMT.addTweak{name = "Classic Ring Attraction", intname = "cra", menu = "gametweaks", options = "boolean", default = false,
attribution = "Golden", -- used to credit people who originally made a tweak
descriptiont = {"Using shields that attract rings, the way they're attracted ", "is more like the classic 2D Sonic games!"},
} -- no func needed, it's handled by the lua script itself


MMT.addTweak{name = "Classic Special Stage", intname = "css", menu = "gametweaks", options = "boolean", default = false,
descriptiont = {"Having 50 rings or more by the end of the stage lets you enter the special stage!"},
func = function(key, player, selft)
    if player.rings >= 50 and (player.pflags & PF_FINISHED) and not All7Emeralds(emeralds) and not selft.EnteredCS and not G_IsSpecialStage(gamemap) then
        selft.EnteredCS = true
        --S_StopSoundByID(player.mo, sfx_token)
        local token = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_TOKEN)
        token.GottaGo = true
    end
end
}

--skincolors[player[entry.prop]].name

local characternames = {"off"}

for i = 0, #skins-1 do
    --skincolornames[i] = skincolors[i].name
    table.insert(characternames, skins[i].realname)
end

-- KNOWN BUG: launching SRB2 with "-file" and loading MMT with this causes the game to not actually register there being any characters, causing errors.
-- i don't know what the fuck causes this, i might report this to the SRB2 gitlab.

MMT.addTweak{name = "Change Character", intname = "changechar", menu = "customplayer", options = "number", default = 0, range = {0, #skins}, names = characternames, dontsave = true,
descriptiont = {"Change the character you're playing as!"},
func = function(key, player, selft)
    if player.mo.skin != skins[player.mmtSettings.changechar-1].name then
        R_SetPlayerSkin(player, skins[player.mmtSettings.changechar-1].name)
        player.mo.color = skins[player.mo.skin].prefcolor
        player.skincolor = player.mo.color
    end
end
}

local skincolornames = {"off"}

for i = 1, #skincolors-1 do
    --skincolornames[i] = skincolors[i].name
    table.insert(skincolornames, skincolors[i].name)
end


MMT.addTweak{name = "Character Color", intname = "charactercolor", menu = "customplayer", options = "bignumber", default = 0, range = {0, #skincolors-1}, names = skincolornames, nonetplay = true,
descriptiont = {"Change your character's colors!"},
func = function(key, player, selft)
    if player.skincolor != player.mmtSettings.charactercolor then
        player.mo.color = player.mmtSettings.charactercolor
        player.skincolor = player.mmtSettings.charactercolor
    end
end
}

addHook("PlayerSpawn", function(player)
    if player.bot then return end

    if not player.mmt then
        player.mmt = {}
        player.mmtSettings = {}
        player.mmtStorage = {} -- used to store important variables related to tweaks
        player.mmtMenu = {}
    end

    player.prevdrawangle = nil

    player.mmtMenu.flashing = 0
    player.mmtMenu.previousmenus = {}

    for key, value in pairs(MMT.TweaksTable) do
        if not value.dontresetme or not player.mmtStorage[value.intname] then
            player.mmtStorage[value.intname] = {}
        else
            player.mmtStorage[value.intname].respawn = true -- done so tweaks can know the player respawned
        end
        -- set defaults before loading player stuff
        if not player.mmtLoaded then
            if value.default != false then
                player.mmtSettings[value.intname] = value.default
            end
        end
    end

    if not player.mmtLoaded then
        MMT.loadSettings(player)
        player.mmtLoaded = true
    end
end)

addHook("PlayerThink", function(player)
    if player.bot then return end
    if player.mmtbt.custom2 == 1 then
        MMT.encodeSettings(player.mmtSettings, "boolean")
        MMT.encodeSettings(player.mmtSettings, "number")
    end

    for key, value in pairs(player.mmtSettings) do
        local tweak = MMT.TweaksTable[key] -- IMPORTANT NOTE: NEVER MODIFY A VARIABLE HERE!!!!!
        if tweak and value then
            if tweak.func and not tweak.global then
                tweak.func(key, player, player.mmtStorage[key])
            end
        end
    end
end)

addHook("MapLoad", function()
    for key, value in pairs(MMT.TweaksTable) do
        if not value.dontresetme or not MMT.mmtGlobalStore[value.intname] then
            MMT.mmtGlobalStore[value.intname] = {}
        else
            MMT.mmtGlobalStore[value.intname].respawn = true -- even though it's called respawn, it's just the map (re)loading
        end
    end
end)



addHook("ThinkFrame", function()
    for key, value in pairs(MMT.mmtGlobalSet) do
        local tweak = MMT.TweaksTable[key] -- IMPORTANT NOTE: NEVER MODIFY A VARIABLE HERE!!!!!
        if tweak and value then
            if tweak.func and tweak.global then
                tweak.func(key, nil, MMT.mmtGlobalStore[key])
            end
        end
    end
end)

addHook("PreThinkFrame", function()
    for player in players.iterate do
        if player.bot or not player.mmtSettings then return end
        for key, value in pairs(player.mmtSettings) do
            local tweak = MMT.TweaksTable[key] -- IMPORTANT NOTE: NEVER MODIFY A VARIABLE HERE!!!!!
            if tweak and value then
                if tweak.prethinkfunc and not tweak.global then
                    tweak.prethinkfunc(key, player, player.mmtStorage[key])
                end
            end
        end
    end
end)

addHook("PostThinkFrame", function()
    for player in players.iterate do
        if player.bot or not player.mmtSettings then return end
        for key, value in pairs(player.mmtSettings) do
            local tweak = MMT.TweaksTable[key] -- IMPORTANT NOTE: NEVER MODIFY A VARIABLE HERE!!!!!
            if tweak and value then
                if tweak.postthinkfunc and not tweak.global then
                    tweak.postthinkfunc(key, player, player.mmtStorage[key])
                end
            end
        end
    end
end)

--[[COM_AddCommand("mmtSet", function(player, var1, var2)
    if var2 == "true" then
        var2 = true
    elseif var2 == "false" then
        var2 = false
    elseif var2 == "nil" then
        var2 = nil
    else
        var2 = tonumber($)
        --print("number")
    end

    player.mmtSettings[var1] = var2
end)

COM_AddCommand("mmtCheck", function(player, var1)
    print(player.mmtSettings[var1])
end)]]

COM_AddCommand("mmtSave", function(player)
    MMT.saveSettings(player, var1)
end)

COM_AddCommand("mmtLoad", function(player)
    MMT.loadSettings(player)
end)

