--[[ MMT Menu System


PLANS:
    - Import SRB2:CE's menu system into here, or just make a new one for scratch, idk what to do. Maybe a bit of both.
    - Construct the menus by looking at the tweaks table, and injecting them into the respective main menus (i.e. a tweak under "gametweaks" would go under "Game Tweaks" as a menu).
    - Call to reconstruct the menus on map load, to enable add-on support.

    - Menu will be called with "mmtMenu" as it has before, but you can hold down custom3 for 5 seconds to turn it on (you should be able to disable this in settings, make it only pop-up the text for opening the menu after 1 second has passed)
]]


local Menus2Construct =
{{"gametweaks", "Gameplay Tweaks"},
{"misctweaks", "Misc. Tweaks"},
{"enhance", "Enhancements"},
{"sfxstyle", "SFX Style"},
{"customplayer", "Customize Player", "characterchanger"},
{"compattweaks", "Compatibility Options"},
}

MMT.menus = {}

local function deepcopy(o, seen)
  seen = seen or {}
  if o == nil then return nil end
  if seen[o] then return seen[o] end

  local no
  if type(o) == 'table' then
    no = {}
    seen[o] = no

    for k, v in next, o, nil do
      no[deepcopy(k, seen)] = deepcopy(v, seen)
    end
    setmetatable(no, deepcopy(getmetatable(o), seen))
  else -- number, string, boolean, etc
    no = o
  end
  return no
end

function MMT.addMenu(name, tbl)
    --table.sort(tbl, sortByKey("order"))
    MMT.menus[name] = tbl
    for key, value in pairs(tbl) do
        if type(value) == "table" then
            if value.intname then
                intnametable[value.intname] = deepcopy(value)
            end
        end
    end
end

MMT.SetPlayerMenu = function(player, menu, mainmenu, dontinsert)
    --[[if not player.khaos.OptionsLoaded then
        CONS_Printf(player, "Hold on, your settings haven't loaded yet!")
        return
    end]]
    if player.bot then
        return
    end
    if player.mmtMenu.menu and not dontinsert then
        table.insert(player.mmtMenu.previousmenus, {player.mmtMenu.menu, player.mmtMenu.page, player.mmtMenu.optionchosen})
    end

    player.mmtMenu.menu = menu
    player.mmtMenu.mainmenu = mainmenu or $
    --player.mmtMenu.type = MMT.menus[player.mmtMenu.menu].type
    player.mmtMenu.page = 1
    player.mmtMenu.optionchosen = 1
    player.mmtMenu.inactive = nil
end

MMT.addMenu("mainmenu",
{
    [1] = {
        name = "Gameplay Tweaks",
        description = "Change your core experience w/ SRB2!",
        table = "gametweaks",
        graphic = "MMT_GAMET"
    },
    [2] = {
        name = "Enhancements",
        description = "Give some flair to how you experience the game!",
        table = "enhance",
        graphic = "MMT_ENHANCE"
    },
    [3] = {
        name = "SFX Style",
        description = "Change your player's sound effects!",
        table = "sfxstyle",
        graphic = "MMT_SOUND"
    },
    [4] = {
        name = "Customize Player",
        description = "Change your character('s properties).",
        table = "customplayer",
        graphic = "MMT_CUSTOM",
        nonetplay = true,
    },
    [5] = {
        name = "Misc. Options",
        description = "Stuff that doesn't fit anywhere else goes here.",
        table = "misctweaks",
        graphic = "MMT_MISC"
    },
    [6] = {
        name = "Compatibility Tweaks",
        description = "Have custom characters that are misbehaving? Try this menu.",
        table = "compattweaks",
        graphic = "MMT_COMPAT",
    },
    [7] = {
        name = "Attributions",
        description = "View the credits for everyone who brought this together!",
        table = "attributionsmenu",
        graphic = "MMT_ATTR",
    },
    type = "catalog",
    name = "\x89" + "M" + "\x80" + "any " + "\x89" + "M" + "\x80" + "inor " + "\x89" + "T" + "\x80" + "weaks",
    subtitlet = {"Use jump to enter a menu, and spin to go back a menu."}
})

local function constructMenus()
    for i = 1, #Menus2Construct do
        local MenuThing = Menus2Construct[i]
        --print(MenuThing[1])
        local indexer = 1
        local InjectMe = {type = MenuThing[3] or "settings", name = MenuThing[2]}
        for index, v2 in ipairs(MMT.TweaksTableINTNAME) do
            local value = MMT.TweaksTable[v2]
            if value.menu == MenuThing[1] then
                local insertion = {
                    name = value.name,
                    option2change = value.intname, -- TODO: reimplement command2change, i'll need it for admin commands!
                    default = value.default,
                    descriptiont = value.descriptiont,
                    graphic = value.graphic,
                    values = value.range,
                    type = value.options,
                    optionnames = value.names,
                    subsetting = value.subsetting,
                    table = value.table,
                    nonetplay = value.nonetplay,
                    admin = value.global,
                }
                InjectMe[indexer] = insertion
                indexer = $ + 1
            end
        end

        MMT.addMenu(MenuThing[1], InjectMe)
    end
end

constructMenus()

local function constructattributionsmenu()
    if insertedblacklisttable then return end
    local thetable1 = {name = "Attributions",}
    local indexer1 = 0
    for key, value in pairs(MMT.attributions) do
        indexer1 = $ + 1
        local insertion = {
            --order = indexer,
            name = value[1],
            order = indexer1,
            description = value[2],
            description2 = value[3],
            graphic = value[4],
            scale = value[5]
        }
        table.insert(thetable1, deepcopy(insertion))
    end

    MMT.addMenu("attributionsmenu", thetable1)
end

constructattributionsmenu()

addHook("MapLoad", function()
    constructMenus()
end)

local function MenuHandler(player, thechosenone, currentmenu)
    if thechosenone.table then
        MMT.SetPlayerMenu(player, thechosenone.table)
        S_StartSound(nil, thechosenone.sound or sfx_menu1, player)
        return true
    end

    if thechosenone.option2change then
        local currentvalue = player.mmtSettings[thechosenone.option2change]
        if thechosenone.admin then
            currentvalue = MMT.mmtGlobalSet[thechosenone.option2change]
        end
        if thechosenone.type == "boolean" then
            if currentvalue then
                currentvalue = nil
            else
                currentvalue = true
            end
            S_StartSound(nil, thechosenone.sound or sfx_menu1, player)
            if thechosenone.admin then
                --print(currentvalue)
                MMT.mmtGlobalSet[thechosenone.option2change] = currentvalue
            else
                player.mmtSettings[thechosenone.option2change] = currentvalue
            end
        elseif thechosenone.type == "number" or thechosenone.type == "bignumber" then
            player.mmtMenu.optionchanger = thechosenone.option2change -- switch modes
            player.mmtMenu.optionchangervalues = thechosenone.values
            S_StartSound(nil, thechosenone.sound or sfx_menu1, player)
        end
        return true
    end

    if currentmenu.type == "warp" then
        if tonumber(CV_FindVar("SRB2CE_Menu_Music").value) then
            S_ChangeMusic("OVWRT", nil, player, nil, S_GetMusicPosition()) --, 250, 250
        end
        local thewarper = thechosenone.warp
        if thewarper == 6969 then
            thewarper = player.khaossp.previouslevel or 1100 -- fall back to 1100 (title screen) if we don't have a level.
        end
        player.khaossp.previouslevel = gamemap
        G_SetCustomExitVars(thewarper, 2)
        G_ExitLevel()
        S_StartSound(nil, thechosenone.sound or sfx_menu1, player)
        return true
    elseif currentmenu.type == "shop" then
        player.khaossp[thechosenone.intname] = true
        player.khaossp.bank = $ - thechosenone.price
        player.khaossp.ringspent = ($ and $ + thechosenone.price) or thechosenone.price
        S_StartSound(nil, thechosenone.sound or sfx_chchng, player)
        return true
    end
end

addHook("PlayerThink", function(player)
    if player.bot then return end
    --[[if player.mmtbt.custom1 == 1 then
        for key, value in pairs(MMT.menus["mainmenu"]) do
            if type(value) == "table" then
                print(key)
                print(value.name)
            end
        end
    end]]

    -- menu logic TWO --
    if player.mmtMenu.flashing > 0 then
        player.mmtMenu.flashing = $ - 1
    end

    if player.mmtMenu.inactive then
        player.mmtMenu.inactive = $ - 1
        if player.mmtMenu.inactive <= 0 then
            player.mmtMenu.menu = nil
            player.mmtMenu.inactive = nil
        end
    end

    if not player.mmtMenu.inactive and player.mmtMenu.menu then
        local currentmenu = MMT.menus[player.mmtMenu.menu] -- DO NOT MODIFY THIS VARIABLE. AT ALL.
        player.pflags = $|PF_FULLSTASIS


        if not player.mmtMenu.optionchanger then
            if player.mmtbt.down == 1 or (player.mmtbt.down > 20 and (leveltime % 2 == 0)) then
                player.mmtMenu.optionchosen = $ + 1
                S_StartSound(nil, sfx_menu1, player)
            end
            if player.mmtbt.up == 1 or (player.mmtbt.up > 20 and (leveltime % 2 == 0)) then
                player.mmtMenu.optionchosen = $ - 1
                S_StartSound(nil, sfx_menu1, player)
            end
        end

        local maxitems = currentmenu.maxitems or 6

        if #currentmenu > maxitems and not player.mmtMenu.optionchangerthen -- we can do pages
            if not player.mmtMenu.optionchanger then
                if player.mmtbt.right == 1 or (player.mmtbt.right > 20 and (leveltime % 2 == 0)) then
                    player.mmtMenu.page = $ + 1
                    player.mmtMenu.optionchosen = 90000 -- force it into the max position
                    S_StartSound(nil, sfx_menu1, player)
                end
                if player.mmtbt.left == 1 or (player.mmtbt.left > 20 and (leveltime % 2 == 0)) then
                    player.mmtMenu.page = $ - 1
                    S_StartSound(nil, sfx_menu1, player)
                end
            end

            local slumber = #currentmenu
            local glumber = 0
            repeat
                slumber = $ - maxitems
                glumber = $ + 1
            until slumber <= 0

            if player.mmtMenu.page > glumber then
                player.mmtMenu.page = 1
            end
            if player.mmtMenu.page < 1 then
                player.mmtMenu.page = glumber
            end
        end

        local minvalue = maxitems*(player.mmtMenu.page-1)+1
        local maxvalue = min(maxitems*player.mmtMenu.page, #currentmenu)

        if player.mmtMenu.optionchosen > maxvalue then
            player.mmtMenu.optionchosen = minvalue
        end
        if player.mmtMenu.optionchosen < minvalue then
            player.mmtMenu.optionchosen = maxvalue
        end

        if player.mmtbt.jump == 1 and not player.mmtMenu.lock then
            if not player.mmtMenu.optionchanger then
                local thechosenone = currentmenu[player.mmtMenu.optionchosen]
                local passrequirements = true

                if thechosenone.admin and netgame and not IsPlayerAdmin(player) then
                    passrequirements = false
                end

                if thechosenone.nonetplay and netgame then
                    passrequirements = false
                end

                if thechosenone.subsetting and not player.mmtSettings[thechosenone.subsetting] then
                    passrequirements = false
                end

                if passrequirements then
                    local thing = MenuHandler(player, thechosenone, currentmenu)
                    if not thing then
                        --CONS_Printf(player, "SOMETHING WENT WRONG, WE'RE NOT DOING ANYTHING!!!!")
                        player.mmtMenu.flashing = 10
                        S_StartSound(nil, sfx_s3k7b, player)
                    end
                else
                    player.mmtMenu.flashing = 15
                    S_StartSound(nil, sfx_cdpcm9, player)
                end
            else
                player.mmtMenu.optionchanger = nil
                player.mmtMenu.optionchangervalues = nil
                S_StartSound(nil, sfx_chchng, player)
            end
        end

        if player.mmtMenu.optionchanger then --TODO: replace this with the relevant mmtSettings stuff
            local thevalueinquestion = player.mmtSettings[player.mmtMenu.optionchanger] --TODO: make this work w/ global variables

            if player.mmtbt.right == 1 or (player.mmtbt.right > 20 and (player.mmtbt.right % 2 == 0)) then
                thevalueinquestion = $ + 1
                S_StartSound(nil, sfx_menu1, player)
            end
            if player.mmtbt.left == 1 or (player.mmtbt.left > 20 and (player.mmtbt.left % 2 == 0)) then
                thevalueinquestion = $ - 1
                S_StartSound(nil, sfx_menu1, player)
            end

            --player.mmtMenu.optionchangervalues[1] -- min
            --player.mmtMenu.optionchangervalues[2] -- max

            if thevalueinquestion > player.mmtMenu.optionchangervalues[2] then
                thevalueinquestion = player.mmtMenu.optionchangervalues[1]
            end
            if thevalueinquestion < player.mmtMenu.optionchangervalues[1] then
                thevalueinquestion = player.mmtMenu.optionchangervalues[2]
            end

            player.mmtSettings[player.mmtMenu.optionchanger] = thevalueinquestion
           -- COM_BufInsertText(consoleplayer, player.mmtMenu.optionchanger + " " + thevalueinquestion)
           --TODO: if desyncs happen, use the above but make it use "mmtSet" instead
        end

        if player.mmtbt.spin == 1 then
            if player.mmtMenu.optionchanger then
                player.mmtMenu.optionchanger = nil
                player.mmtMenu.optionchangervalues = nil
                S_StartSound(nil, sfx_chchng, player)
            elseif player.mmtMenu.menu != player.mmtMenu.mainmenu then
                local menuset = {player.mmtMenu.mainmenu, 1, 1} -- fallback
                --player.mmtMenu.previousmenus
                if #player.mmtMenu.previousmenus >= 1 then
                    menuset = player.mmtMenu.previousmenus[#player.mmtMenu.previousmenus]
                    table.remove(player.mmtMenu.previousmenus, #player.mmtMenu.previousmenus)
                end
                MMT.SetPlayerMenu(player, menuset[1], nil, true)
                player.mmtMenu.page, player.mmtMenu.optionchosen = menuset[2], menuset[3]
                S_StartSound(nil, sfx_menu1, player)
            else
                --player.mmtMenu.menu = nil
                player.mmtMenu.inactive = TICRATE * 2
                player.mmtMenu.previousmenus = {} -- force purge

                MMT.saveSettings(player)
                S_ChangeMusic(mapmusname, true, player) -- mapheaderinfo[gamemap].musname
                S_StartSound(nil, sfx_cdpcm9, player)
            end
        end

    end
end)

hud.add(function(v, player)
    if player.mmtSettings.amwb then -- button 2 open the mmtMenu
        local Goal = (TICRATE) + (TICRATE/4)
        if v.buttontimer == nil then
            v.buttontimer = -10
        end

        local buttomtime = player.mmtbt[MMT.mmtKeys[player.mmtSettings.amwb + 1]]

        if buttomtime > 0 and not player.mmtMenu.menu then
            v.buttontimer = min($ + 1, Goal)
        else
            v.buttontimer = max($ - 1, -10)
        end

        local percenttime = FRACUNIT*(v.buttontimer) / (Goal)
        local thesmalloffset = ease.outexpo(percenttime, 80*FRACUNIT, 0)
        local timertext = abs(min((buttomtime/TICRATE), 4) - 4)

        Fontlib.drawString(v, 160, 180+(thesmalloffset/FRACUNIT), "Hold \x81" + MMT.mmtKeys[player.mmtSettings.amwb + 1] + "\x80 for \x82" + timertext + "\x80 seconds to open MMT menu.", {font = "KHFNT"}, "center")
    end

    if not player.mmtMenu or not player.mmtMenu.menu then v.mmtminit = false return end

    local currentmenu = MMT.menus[player.mmtMenu.menu] -- DO NOT MODIFY THIS VARIABLE. AT ALL.

    if not v.mmtminit then
        v.mmtminit = true
        v.MMTmTrans = 9
        v.mmtmtimer = 0
        v.mmtmtimer2 = 0
    end

    if not player.mmtMenu.inactive then
        if (leveltime % 3 == 0) and v.MMTmTrans > 2 then
            v.MMTmTrans = $ - 1
        end
    else
        if (leveltime % 5 == 0) and v.MMTmTrans < 9 then
            v.MMTmTrans = $ + 1
        end
    end

    v.TransFlags = v.MMTmTrans<<V_ALPHASHIFT
    if v.MMTmTrans < 9 then
        v.drawScaled(0, 200, FRACUNIT*800, v.cachePatch("THEYHAVEIT"), v.TransFlags|V_SNAPTOLEFT|V_SNAPTOTOP) -- square :)
    end

    local slumber = #currentmenu
    local glumber = 0
    local maxitems = currentmenu.maxitems or 6
    repeat
        slumber = $ - maxitems
        glumber = $ + 1
    until slumber <= 0


    local thesmalloffset
    local TheBigOffset
    local TheBigOffset2

    if not player.mmtMenu.inactive then
        local Goal = (TICRATE * 2) - (TICRATE/2)
        v.mmtmtimer = min($ + 1, Goal)

        local percenttime = FRACUNIT*(v.mmtmtimer) / (Goal)

        thesmalloffset = ease.outexpo(percenttime, 80*FRACUNIT, 0)
        TheBigOffset = ease.outexpo(percenttime, v.width()/v.dupx()*FRACUNIT, 0)
        TheBigOffset2 = ease.outexpo(percenttime, -v.width()/v.dupx()*FRACUNIT, 0)
    else
        local Goal = (TICRATE) + (TICRATE/4)
        v.mmtmtimer2 = min($ + 1, Goal)

        local percenttime = FRACUNIT*(v.mmtmtimer2) / (Goal)

        thesmalloffset = ease.inback(percenttime, 0, 80*FRACUNIT)
        TheBigOffset = ease.inback(percenttime, 0, v.width()/v.dupx()*FRACUNIT)
        TheBigOffset2 = ease.inback(percenttime, 0, -v.width()/v.dupx()*FRACUNIT)
    end


    local subtitleoffset = 0
    if currentmenu.subtitlet then
        for index, value in ipairs(currentmenu.subtitlet) do -- TODO: this is currently broken, but i'm too lazy to fix it right now. it only supports 1 line, more than 1 and it fucks up
            Fontlib.drawString(v, 160, ((index)*5)+30-(thesmalloffset/FRACUNIT), value, {font = "KHFNT"}, "center")
            subtitleoffset = $ - 5
        end
    end

    Fontlib.drawString(v, 160, subtitleoffset+26-(thesmalloffset/FRACUNIT), tostring(currentmenu.name), {font = "KHMNT"}, "center") -- "\x83" +

    --if glumber > 1 then v.drawString(120, 32, "Page: " + (player.mmtMenu.page + "/" + glumber) , V_ALLOWLOWERCASE, "thin-center") end
    if glumber > 1 then
        local leftarrow = " "
        local rightarrow = " "
        if player.mmtMenu.page > 1 then
            leftarrow = "\x82<-\x80"
        end
        if player.mmtMenu.page < glumber then
            rightarrow = "\x82->"
        end
        v.drawString(160, 190+(thesmalloffset/FRACUNIT), leftarrow + "Page: " + (player.mmtMenu.page + "/" + glumber) + rightarrow , V_ALLOWLOWERCASE, "thin-center")

    end


    local maxitems = currentmenu.maxitems or 6

    local minvalue = maxitems*(player.mmtMenu.page-1)+1
    local maxvalue = min(maxitems*player.mmtMenu.page, #currentmenu)

    local ything = 0
    local bindex = 1
    for key, value in pairs(currentmenu) do
        if key != "subtitlet" and type(value) == "table" and ((key >= minvalue) and (key <= maxvalue)) then
            -- if current option is this one, we put the cursor over it
            local pretext = ""

            local gabel = -- this is done so we can modify what we actually display
            {
                ["graphic"] = value.graphic,
                ["graphic2"] = value.graphic2,
                ["scale"] = value.scale,
                ["scale2"] = value.scale2,
                ["xoffset"] = value.xoffset,
                ["yoffset"] = value.yoffset,
                ["xoffset2"] = value.xoffset2,
                ["yoffset2"] = value.yoffset2,
                ["name"] = value.name,
                ["subtitlet"] = value.subtitlet,
                ["description"] = value.description,
                ["description2"] = value.description2,
                ["descriptiont"] = value.descriptiont,
            }

            local passrequirements = true
            local requirestring = ""
            local price
            local subpretext = "\x80"

            if value.subsetting and not player.mmtSettings[value.subsetting] then
                passrequirements = false
            end

            if value.admin and (netgame and not IsPlayerAdmin(player)) then
                passrequirements = false
            end

            if value.nonetplay and (netgame) then
                passrequirements = false
            end

            if value.option2change then
                local setto = player.mmtSettings[value.option2change]
                if value.admin then
                    setto = MMT.mmtGlobalSet[value.option2change]
                end
                if value.type == "boolean" then
                    if not setto then
                        setto = "off"
                    else
                        setto = "on"
                    end
                end
                if value.optionnames then
                    setto = value.optionnames[setto+1]
                end
                price = "\x82Set to: \x80" + setto
            end

            if player.mmtMenu.optionchosen == key then
                pretext = "\x83"
                if player.mmtMenu.optionchanger then
                    pretext = "\x82"
                    price = "<-- " + $ + " -->"
                end
            end

            if value.animated then
                local number = (leveltime/value.speed) % value.frames
                local extra = tostring((number % 100 < 10 and "0") or "")
                gabel.graphic = $ + extra + tostring(number)
            end

            if not passrequirements then
                gabel.descriptiont = nil
                --name = "???"
                pretext = "\x86"
                if player.mmtMenu.optionchosen == key then
                    pretext = "\x85"
                end
                subpretext = "\x86"
                local typer = "use"

                if value.subsetting and not player.mmtSettings[value.subsetting] then
                    gabel.description = "Before you can change this, you need to enable:"
                    gabel.description2 = MMT.TweaksTable[value.subsetting].name
                    price = nil
                end

                if value.nonetplay then
                    gabel.description = "You can't set this while in a netgame!"
                    gabel.description2 = nil
                    price = nil
                else
                    if (netgame and not IsPlayerAdmin(player)) then
                        gabel.description = "You need to be an admin to set this!"
                        gabel.description2 = nil
                        price = nil
                    end
                end
            end

            if currentmenu.type == "shop" and not value.price then
                gabel.description = value.description
                gabel.description2 = value.description2
            end

            if currentmenu.type == "warp" then
                local mapnum = value.warp
                if value.warp == 6969 then
                    mapnum = player.khaossp.previouslevel or 1000
                    if player.khaossp.previouslevel then
                        gabel.description2 = "\x82" + "Warp to: \x80" + G_BuildMapTitle(player.khaossp.previouslevel)
                    else
                        gabel.name = "Go back to Title Screen" -- naming it something different, hopefully to catch the player's attention a bit
                        gabel.description2 = "\x82" + "Warp to:\x80 Title Screen"
                    end
                end
                gabel.graphic = G_BuildMapName(mapnum) + "P"
                if not v.patchExists(gabel.graphic) then
                    gabel.graphic = "MAPZ0P" -- fallback for in case
                end
                gabel.mapgraphic = true
            end

            if currentmenu.type == "unlockstatistics" then
                local unlocked = player.khaossp[value.savevalue]
                if unlocked then
                    price = "\x83\Unlocked"
                else
                    price = "\x85\Not found yet!"
                end
            end

            if currentmenu.type == "statistics" then
                price = "\x83" + player.khaossp[value.savevalue]
                if value.type == "timer" then
                    local timer = player.khaossp[value.savevalue]
                    local hours = (timer / 3600)
                    local minutes = (timer / 60) % 60
                    local seconds = timer % 60
                    if seconds < 10 then
                        seconds = "0" .. tostring($)
                    end
                    if minutes < 10 then
                        minutes = "0" .. tostring($)
                    end

                    --local stringer = tostring(seconds)
                    -- convert to hh:mm:ss

                    price = "\x83" + tostring(hours) + ":" + tostring(minutes) + ":" + tostring(seconds)
                end

                if value.type == "timers" then
                    local timer = player.khaossp[value.savevalue]
                    local seconds = (timer / 35) % 60
                    local minutes = (timer / 35) / 60 -- if you somehow get more than 60 minutes, then fuck you, you don't get an hours counter
                    local miliseconds = ((timer % 35) * 100) / 35
                    if minutes < 10 then
                        minutes = "0" .. tostring($)
                    end
                    if seconds < 10 then
                        seconds = "0" .. tostring($)
                    end
                    if miliseconds < 10 then
                        miliseconds = "0" .. tostring($)
                    end

                    --local stringer = tostring(seconds)
                    -- convert to hh:mm:ss

                    price = "\x83" + tostring(minutes) + ":" + tostring(seconds) + "." + tostring(miliseconds)
                end
            end

            if player.mmtMenu.optionchosen == key --TODO: i've done this 3 times by now, isn't there a simpler way?
            and player.mmtMenu.flashing > 0 and (leveltime % 3 == 0) then
                if pretext == "\x83" then
                    pretext = "\x87"
                else
                    pretext = "\x82"
                end
            end

            local offset2
            if (bindex % 2) == 0 then
                offset2 = TheBigOffset2
            else
                offset2 = TheBigOffset
            end

            if not gabel.mapgraphic then
                if gabel.graphic then
                    v.drawScaled((0+(gabel.xoffset or 0)*FRACUNIT) + offset2, (50+ything+(gabel.yoffset or 0))*FRACUNIT, FRACUNIT/(gabel.scale or 2), v.cachePatch(gabel.graphic))
                end
                if gabel.graphic2 then
                    v.drawScaled((0+(gabel.xoffset2 or 0)*FRACUNIT) + offset2, (50+ything+(gabel.yoffset2 or 0))*FRACUNIT, FRACUNIT/(gabel.scale2 or 2), v.cachePatch(gabel.graphic2))
                end
            else
                local scaler = (FRACUNIT/5 + FRACUNIT/5 + FRACUNIT/25)/2
                v.drawCropped((0+(gabel.xoffset or 0)*FRACUNIT) + offset2, (50+ything+(gabel.yoffset or 0))*FRACUNIT, scaler, scaler, v.cachePatch(gabel.graphic), 0, nil, 58*FRACUNIT, 28*FRACUNIT, 44*FRACUNIT*2, 44*FRACUNIT*2)
            end
            v.drawString(24 + (offset2/FRACUNIT), 50+ything, pretext+gabel.name, V_ALLOWLOWERCASE, "thin")

            local priceoffset = 0
            if gabel.description then v.drawString(24 + (offset2/FRACUNIT), 50+7+ything, subpretext + gabel.description, V_ALLOWLOWERCASE, "small") priceoffset = $ + 5 end
            if gabel.description2 then v.drawString(24 + (offset2/FRACUNIT), 50+7+5+ything, subpretext + gabel.description2, V_ALLOWLOWERCASE, "small") priceoffset = $ + 5 end
            if gabel.descriptiont then
                local theresetter = ything
                for index, value in ipairs(gabel.descriptiont) do
                    v.drawString(24 + (offset2/FRACUNIT), 50+7+ything, subpretext + value, V_ALLOWLOWERCASE, "small")
                    ything = $ + 5
                    priceoffset = $ + 5
                end
                ything = theresetter
            end
            if price then v.drawString(24 + (offset2/FRACUNIT), 50+7+priceoffset+ything, price, V_ALLOWLOWERCASE, "small") end
            ything = $ + (currentmenu.spacing or 17) + 6


            bindex = $ + 1
        end
    end
end, "game")



MMT.MusicList = {"MMTM", "MMT1", "MMT3", "MMT4", "MMT5"}

COM_AddCommand("mmtMenu", function(player)
    --summon the menu!
    MMT.SetPlayerMenu(player, "mainmenu", "mainmenu")

    local musictrack = "MMTM"
    if player.mmtSettings.menum == 0 then
        return
    elseif player.mmtSettings.menum == 1 then
        musictrack = MMT.MusicList[P_RandomRange(1, #MMT.MusicList)]
    else
        musictrack = MMT.MusicList[player.mmtSettings.menum - 1]
    end
    S_ChangeMusic(musictrack, nil, player)
end)


-- !! justtheplayer.lua !! --

hud.add(function(v,player,c)
    if not player or not player.mmtMenu or not player.mmtMenu.menu or MMT.menus[player.mmtMenu.menu].type != "characterchanger" then v.frame = nil return end
    -- player.mmtMenu.menu.type == "characterchanger"
    if v.frame == nil then
        v.frame = 0
        v.rotatetimer = 0
        v.flipper = false
        v.idletimer = 0
        v.falltimer = 0
    end

    local Goal = (TICRATE) - (TICRATE/3)
    v.falltimer = min($ + 1, Goal)

    local percenttime = FRACUNIT*(v.falltimer) / (Goal)

    local thesmalloffset = ease.incubic(percenttime, -100*FRACUNIT, 110*FRACUNIT)

    v.frame = $ + 1
    --v.rotatetimer = $ + 1
    -- commented out because tails would look werid if he rotated lmao, he won't have his tails. if STJR gets around to rendering the tails in the multiplayer menu themselves, then i might.
    local delay = 5
    local Spriter = SPR2_WALK

    if not player.mmtbt.hmoving then
        v.idletimer = $+1
    else
        v.idletimer = 0
    end

    if player.mmtbt.hmoving > 2*TICRATE then
        delay = 2
        Spriter = SPR2_RUN
    end

    if v.idletimer > 4*TICRATE then
        delay = 15
        Spriter = SPR2_WAIT
    end

    if v.falltimer < Goal then
        Spriter = SPR2_FALL
    end

    local Framer = v.frame / delay
    local Rotater = 2 --v.rotatetimer / 12
    if player.followitem or
    player.mo.skin == "fluffy" then -- chrispy thinks he's too cool for follow items! well he's right
        Rotater = 1
    end

    if Rotater > 8 then
        v.flipper = not v.flipper
        v.rotatetimer = 0
        Rotater = 0
    end
    if skins[player.mo.skin].sprites[Spriter].numframes then
        if Framer > skins[player.mo.skin].sprites[Spriter].numframes-1 then
            v.frame = 0
            Framer = v.frame / delay
        end
    else
        v.frame = 0
        Framer = 0
    end

    local character, flipped = v.getSprite2Patch(player.mo.skin, Spriter, false, Framer, Rotater)
    local colormap = v.getColormap(player.mo.skin, player.mo.color)
    local flags = nil
    if flipped == true then
        flags = V_FLIP
    end

    v.drawScaled(160*FRACUNIT, (thesmalloffset) + (70*FRACUNIT), FRACUNIT, character, flags, colormap)
end, "game")

